"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const fs = require("fs");
const constants_1 = require("./constants");
const logger = console;
class GitSentinel {
    constructor() {
        this.changedFiles = [];
        this.errors = [];
        this.statuses = [];
        const config = this.findConfig();
        if (!config) {
            console.log('sentinel.json not found');
            process.exit(1);
        }
        this.settings = config;
    }
    findConfig() {
        let dir = __dirname;
        // console.log({ dir });
        while (dir.length) {
            dir = dir.substr(0, dir.lastIndexOf(constants_1.default.dirDelimiter));
            if (fs.existsSync(`${dir}/sentinel.json`)) {
                const config = JSON.parse(fs.readFileSync(`${dir}/sentinel.json`).toString());
                return config;
            }
        }
    }
    fire() {
        this.findConfig();
        this.changedFiles = this.getChangedFiles();
        this.writeChangedFiles();
        for (const rule of this.settings.rules) {
            const firedFiles = this.conditionMet(this.changedFiles, rule.mask);
            // logger.info('starting rule', rule.name);
            // logger.info('firedFiles', firedFiles);
            if (firedFiles.length) {
                this.fireRule(rule, firedFiles);
            }
        }
        // console.log('\x1b[0m');
        // console.log('Статусы', this.statuses);
        this.writeErrors();
        return this.errors.length ? 1 : 0;
    }
    writeChangedFiles() {
        if (this.changedFiles.length) {
            console.log('\x1b[36m', 'Файлы');
            for (const file of this.changedFiles) {
                process.stdout.write(`- ${file} \n`);
            }
            process.stdout.write('\n\n');
        }
        else {
            console.log('Файлы не были изменены');
        }
    }
    writeErrors() {
        if (this.errors.length) {
            console.log('\x1b[31m', 'Ошибки');
            for (const err of this.errors) {
                process.stdout.write(`${err.cmd} : \n`);
                process.stdout.write(err.message);
                process.stdout.write('\n\n');
            }
            console.log('\x1b[0m');
        }
        else {
            console.log('\x1b[32m', 'Ошибок нет');
            console.log('\x1b[0m');
        }
    }
    conditionMet(changedFiles, mask) {
        return changedFiles.filter((el) => {
            const match = el.match(new RegExp(mask));
            if (match && match.length > 0) {
                return true;
            }
            return false;
        });
    }
    fireRule(rule, files) {
        let filesToFire = files;
        if (!rule.separate) {
            filesToFire = [''];
        }
        const commands = rule.commands;
        for (const file of filesToFire) {
            for (const cmd of commands) {
                const success = this.runCmd(this.getCmdForFilename(file, cmd));
                if (!success && rule.stopOnErrors) {
                    return;
                }
            }
        }
    }
    runCmd(cmd) {
        let message = '';
        let success = true;
        try {
            const cmdRet = child_process_1.execSync(cmd, { stdio: ['pipe', 'pipe', 'ignore'] });
            message = cmdRet.toString().trim();
        }
        catch (err) {
            success = false;
            message = err.output[1].toString().trim();
        }
        this.statuses.push({
            cmd,
            success,
            message,
        });
        this.addStatus(cmd, success, message);
        return success;
    }
    addStatus(cmd, success, message) {
        this.statuses.push({ cmd, success, message });
        if (!success) {
            this.errors.push({ cmd, message });
        }
    }
    /**
     * Возвращает список измененных в этом коммите файлов
     */
    getChangedFiles() {
        const ret = child_process_1.execSync('git diff HEAD --name-only');
        const newFiles = ret.toString().split('\n').filter(e => e);
        return newFiles;
    }
    /**
     * Подставляем название файла в команду, заменяя ${filename} на название файла
     *
     * @param filename - название файла
     * @param cmd - команда
     */
    getCmdForFilename(filename, cmd) {
        return cmd.replace('${filename}', filename);
    }
}
exports.default = GitSentinel;
//# sourceMappingURL=gitSentinel.js.map